@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright (C) 2009 by Daniel Martin Yerga
+# Copyright (C) 2009-2010 by Daniel Martin Yerga
 # <dyerga@gmail.com>
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,10 +17,12 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 #
 # CasualServices: Search and share your favourite places
-# Version 0.4
+# Version 0.5
 #
 
-VERSION = "0.4"
+VERSION = "0.5"
+NAME = "CasualServices"
+URL = "http://cusl4-cservices.forja.rediris.es/"
 
 import gtk
 import hildon
@@ -34,6 +36,7 @@ import gettext
 import osso
 import settings
 import cgi
+import urllib2
 
 try:
     import location
@@ -111,6 +114,7 @@ countries = []
 areas = []
 medialinks = []
 listimg = []
+links = []
 
 latitudes = []
 longitudes = []
@@ -213,8 +217,10 @@ class MainWindow:
         self.radius = '10'
         self.profile = ''
         self.location = ''
+        self.stop_gps = False
         self.settings = settings.Settings()
-        self.mapsource, self.username, self.authtoken, self.profile = self.settings.load_config()
+        self.mapsource, self.username, self.authtoken, self.profile, \
+            self.twittuser, self.twittpass = self.settings.load_config()
 
         #Create the widgets of the main window
         vbox = self.initial_screen()
@@ -296,13 +302,6 @@ class MainWindow:
 
         hbox = gtk.HBox()
 
-        #button = hildon.CheckButton(fhsize)
-        #img = gtk.Image()
-        #img.set_from_file('/usr/share/icons/hicolor/48x48/hildon/camera_gps_location_pressed.png')
-        #button.set_image(img)
-        #button.connect("clicked", self.remove_recipes)
-        #hbox.pack_start(button, False, False, 5)
-
         #Text entry for searches
         self.searchentry = hildon.Entry(fhsize)
         self.searchentry.connect("activate", self.search_services)
@@ -375,11 +374,14 @@ class MainWindow:
         button.connect("clicked", self.show_settings_dlg)
         menu.append(button)
 
+        button = gtk.Button(_("Help"))
+        button.connect("clicked", self.show_help)
+        menu.append(button)
+
         button = gtk.Button(_("About"))
         button.connect("clicked", About)
         menu.append(button)
 
-
         #button = gtk.Button(_("Create new service"))
         #button.connect("clicked", self.create_service)
         #menu.append(button)
@@ -556,7 +558,7 @@ class MainWindow:
         #searchlist = [titles, localities, areas, countries, listimg]
 
         if searchlist:
-            print searchlist[4]
+            #print searchlist[4]
             #If there are search results, add it to the gtk.liststore
             for i in range(len(searchlist[0])):
                 liter = lstore.append()
@@ -579,7 +581,6 @@ class MainWindow:
         @returns: gtk.gdk.Pixbuf with the image from the URL
 
         """
-        import urllib2
         try:
             myimg = urllib2.urlopen(urlimg)
             imgdata = myimg.read()
@@ -637,6 +638,11 @@ class MainWindow:
         button = gtk.Button(_("Add to Contacts"))
         button.connect("clicked", self.add_to_contacts, path)
         menu.append(button)
+
+        button = gtk.Button(_("Share service"))
+        button.connect("clicked", self.share_place, path)
+        menu.append(button)
+
         menu.show_all()
         self.serwin.set_app_menu(menu)
 
@@ -759,6 +765,8 @@ class MainWindow:
         self.horbox = gtk.HBox()
         self.verbox = gtk.VBox()
 
+        #Check if the orientation is portrait or landscape
+        #Depending on the orientation, the layout will be different
         if self.is_portrait():
             self.serparea.set_size_request(-1, 480)
             self.verbox.pack_start(self.serparea, False, False, 0)
@@ -777,7 +785,6 @@ class MainWindow:
         except ValueError:
             pass
 
-
         self.add_pixmap_to_map(lat, lon)
 
         if medialinks[path[0]]:
@@ -842,6 +849,42 @@ class MainWindow:
 
         if result:
             self.show_info_banner(widget, _("Service added to Contacts"))
+        else:
+            self.show_info_banner(widget, _("Error adding service to Contacts"))
+
+    def share_place(self, widget, path):
+        dlg = hildon.PickerDialog(self.window)
+        dlg.set_title(_("Where to post"))
+        data = ["Twitter"]
+        selector = self.create_selector(data)
+        selector.connect("changed" , self.select_share_service, dlg, path)
+        dlg.set_selector(selector)
+
+        dlg.show_all()
+        dlg.run()
+
+    def select_share_service(self, selector, column, dlg, path):
+        value = selector.get_current_text()
+        dlg.destroy()
+        if value == 'Twitter':
+            winprogind(self.serwin, 1)
+            thread.start_new(self.share_on_twitter, (path,))
+
+    def share_on_twitter(self, path):
+        import twitter
+        try:
+            api = twitter.Api(username=self.twittuser, password=self.twittpass)
+            short = urllib2.urlopen("http://is.gd/api.php?longurl=%s" % urllib2.quote(links[path[0]])).read()
+
+            text = "I'm at %s, %s" % (titles[path[0]], short)
+
+            status = api.PostUpdate(text)
+            winprogind(self.serwin, 0)
+            self.show_banner_thread(self.serwin, _('Service posted!'))
+        except:
+            logger.exception('sharing to twitter')
+            winprogind(self.serwin, 0)
+            self.show_banner_thread(self.serwin, _('Error sharing service'))
 
     def is_portrait(self):
         """Check if the window is in portrait
@@ -867,7 +910,7 @@ class MainWindow:
         """
 
         if not self.map.get_parent_window():
-            print 'not in the right screen, doing nothing'
+            #print 'not in the right screen, doing nothing'
             return
 
         try:
@@ -941,10 +984,14 @@ class MainWindow:
         """
         self.activate_gps(widget)
 
-        #FIXME: wait for a real GPS fix to show the position in the map
+        #while not self.gps.has_fix:
+        #    pass
+
+        #lat, lon = self.gps.latlon
+
 
         lat, lon = self.gps.get_data()
-        print lat, lon
+        #print lat, lon
 
         #Draw our position in the map
         themap.draw_gps(float(lat), float(lon), 1.0)
@@ -1018,10 +1065,31 @@ class MainWindow:
                 #Activate the GPS to check your position
                 self.activate_gps(widget)
                 lat, lon = self.gps.get_data()
-                #FIXME: GPS stuff
+
+                #print 'has fix: ', self.gps.has_fix
+
+                #if not self.gps.has_fix:
+                 #   gtk.gdk.threads_enter()
+                 #   fixdlg = self.gpsfix_dialog()
+                 #   gtk.gdk.threads_leave()
+
+                 #   if fixdlg:
+                 #       lat, lon = self.gps.latlon
+                #        print lat, lon
+                #    else:
+                #        print 'stopping function'
+                #        winprogind(self.window, 0)
+                #        return
+
+
+               # while not self.gps.has_fix:
+               #     pass
+
+                #lat, lon = self.gps.latlon
+                print lat, lon
                 #lat = '43.535000'
                 #lon = '-5.942500'
-                #feed = self.onceAPI.searchgeo(text, self.page, lat, lon, self.radius)
+                feed = self.onceAPI.searchgeo(text, self.page, lat, lon, self.radius)
                 #feed = {'entries': {}}
                 glocalresults = self.GLocal.searchgeo(text, self.page, lat, lon, self.radius)
             #If search for location has a text defined
@@ -1125,85 +1193,97 @@ class MainWindow:
         del areas[:]
         del medialinks[:]
         del listimg[:]
+        del links[:]
 
         #if some data is not available, show this
         nodata = _('Not available')
 
-        #Parse the dictionary and add the data to the different lists
-        for i in feed['entries']:
-            if i.has_key('title'):
-                titles.append(i['title'])
-            else:
-                titles.append(nodata)
+        try:
+            #Parse the dictionary and add the data to the different lists
+            for i in feed['entries']:
+                if i.has_key('title'):
+                    titles.append(i['title'])
+                else:
+                    titles.append(nodata)
 
-            if i.has_key('telephone'):
-                phones.append(i['telephone'])
-            else:
-                phones.append(nodata)
+                if i.has_key('link'):
+                    links.append(i['link'])
+                else:
+                    links.append('')
 
-            if i.has_key('url'):
-                urls.append(i['url'])
-            else:
-                urls.append(nodata)
+                if i.has_key('telephone'):
+                    phones.append(i['telephone'])
+                else:
+                    phones.append(nodata)
 
-            if i.has_key('locality'):
-                localities.append(i['locality'])
-            else:
-                localities.append(nodata)
+                if i.has_key('url'):
+                    urls.append(i['url'])
+                else:
+                    urls.append(nodata)
 
-            if i.has_key('pos'):
-                gmlpos.append(i['pos'])
-            else:
-                gmlpos.append(nodata)
+                if i.has_key('locality'):
+                    localities.append(i['locality'])
+                else:
+                    localities.append(nodata)
 
-            if i.has_key('summary'):
-                summaries.append(i['summary'].replace('&', '&amp;'))
-            else:
-                summaries.append(nodata)
+                if i.has_key('pos'):
+                    gmlpos.append(i['pos'])
+                else:
+                    gmlpos.append(nodata)
 
-            if i.has_key('useraddress'):
-                addresses.append(i['useraddress'])
-            else:
-                addresses.append(nodata)
+                if i.has_key('summary'):
+                    summaries.append(i['summary'].replace('&', '&amp;'))
+                else:
+                    summaries.append(nodata)
 
-            if i.has_key('country'):
-                countries.append(i['country'])
-            else:
-                countries.append(nodata)
+                if i.has_key('useraddress'):
+                    addresses.append(i['useraddress'])
+                else:
+                    addresses.append(nodata)
 
-            if i.has_key('subadministrativearea'):
-                areas.append(i['subadministrativearea'])
-            else:
-                areas.append(nodata)
+                if i.has_key('country'):
+                    countries.append(i['country'])
+                else:
+                    countries.append(nodata)
 
-            mlinks = []
-            if i.has_key('links'):
-                for j in i['links']:
-                    if j['rel'] == 'media':
-                        if 'ps_' in j['href']:
-                            if "http://11870.com" in j['href']:
-                                mlinks.append(j['href'])
-                            else:
-                                mlinks.append('http://11870.com/%s' % j['href'])
-                    #else:
-                    #    print 'normal url'
-                medialinks.append(mlinks)
+                if i.has_key('subadministrativearea'):
+                    areas.append(i['subadministrativearea'])
+                else:
+                    areas.append(nodata)
 
-            else:
-                medialinks.append(nodata)
+                mlinks = []
+                if i.has_key('links'):
+                    for j in i['links']:
+                        if j['rel'] == 'media':
+                            if 'ps_' in j['href']:
+                                if "http://11870.com" in j['href']:
+                                    mlinks.append(j['href'])
+                                else:
+                                    mlinks.append('http://11870.com/%s' % j['href'])
+                        #else:
+                        #    print 'normal url'
+                    medialinks.append(mlinks)
 
-        for i in medialinks:
-            try:
-                listimg.append(i[0])
-            except:
-                listimg.append('')
+                else:
+                    medialinks.append(nodata)
+
+            for i in medialinks:
+                try:
+                    listimg.append(i[0])
+                except:
+                    listimg.append('')
 
 
-        self.parse_google_search(googlecontent)
+            self.parse_google_search(googlecontent)
 
-        searchlist = [titles, localities, areas, countries, listimg]
+            searchlist = [titles, localities, areas, countries, listimg]
+            return searchlist
 
-        return searchlist
+        except:
+            logger.exception('parsing search results')
+            self.show_info_banner(widget, _("Error parsing search results"))
+            searchlist = [titles, localities, areas, countries, listimg]
+            return searchlist
 
     def parse_google_search(self, content):
         """Add Google Local results to the service details lists
@@ -1222,11 +1302,16 @@ class MainWindow:
                 titles.append(nodata)
 
             if i.has_key('phoneNumbers'):
-                print i['phoneNumbers'][0]['number']
+                #print i['phoneNumbers'][0]['number']
                 phones.append(i['phoneNumbers'][0]['number'])
             else:
                 phones.append(nodata)
 
+            if i.has_key('url'):
+                links.append(i['url'])
+            else:
+                links.append('')
+
             #TODO: add some URL
             #if i.has_key(''):
             #    urls.append(i[''])
@@ -1285,7 +1370,7 @@ class MainWindow:
                 tile_cache = HOME + '/MyDocs/.maps')
 
         themap.add_layer(osmgpsmap.GpsMapOsd())
-        themap.connect('button_press_event', self.map_clicked)
+        themap.connect('button_press_event', self.map_clicked, win)
 
         vbox.pack_start(themap, True, True, 0)
 
@@ -1330,7 +1415,7 @@ class MainWindow:
 
         for coord in gmlpos:
             lat, lon = coord.split(' ')
-            if coord != 'Not available':
+            if coord != _('Not available'):
                 latitudes.append(lat)
                 longitudes.append(lon)
             else:
@@ -1363,7 +1448,7 @@ class MainWindow:
         except Exception, e:
             print 'Failed to center map: ', e
 
-    def map_clicked(self, themap, event):
+    def map_clicked(self, themap, event, win):
         """Do actions when clicking in a map.
         Single click: check if the click is in a service
         Double click: zoom in the map
@@ -1375,16 +1460,20 @@ class MainWindow:
 
         """
         import math
+        rlat, rlon = themap.get_co_ordinates(event.x, event.y)
+        rlat = math.degrees(rlat)
+        rlon = math.degrees(rlon)
+
         if event.button == 1:
             if event.type == gtk.gdk._2BUTTON_PRESS:
-                themap.set_zoom(themap.props.zoom + 3)
+                themap.set_mapcenter (float(rlat), float(rlon),
+                                        themap.props.zoom + 3)
             elif event.type == gtk.gdk.BUTTON_PRESS:
-                rlat, rlon = themap.get_co_ordinates(event.x, event.y)
                 #print math.degrees(rlat)
                 #print math.degrees(rlon)
-                self.check_if_tap_service(math.degrees(rlat), math.degrees(rlon), themap)
+                self.check_if_tap_service(rlat, rlon, themap, win)
 
-    def check_if_tap_service(self, taplat, taplon, themap):
+    def check_if_tap_service(self, taplat, taplon, themap, win):
         """Check if a click in the map is in a service
 
         @type   taplat: string
@@ -1428,18 +1517,19 @@ class MainWindow:
                 #print 'nj: ', nj
                 if nj == taplon:
                     #print 'a service selected'
-                    self.show_service_info_dialog(i)
+                    self.show_service_info_dialog(i, win)
                     return
                 break
 
-    def show_service_info_dialog(self, index):
+    def show_service_info_dialog(self, index, win):
         """Show a dialog with information about a service
 
         @type   index: number
         @param  index: Index of the service in the results list
 
         """
-        dlg = gtk.Dialog(title=_('Show service information'), parent=None, flags=0)
+        dlg = gtk.Dialog(title=_('Show service information'), parent=win, flags=0)
+        dlg.set_transient_for(win)
         dlg.set_has_separator(False)
 
         label = gtk.Label()
@@ -1508,6 +1598,16 @@ class MainWindow:
         """
         self.gps = GPS()
 
+    #def gpsfix_dialog(self):
+    #    dlg
+    #
+    #    result = dlg.run()
+    #    if result == Stop:
+    #        return False
+    #
+    #    elif self.gps.has_fix
+
+
     def show_settings_dlg(self, widget):
         """Show the settings dialog
 
@@ -1515,27 +1615,18 @@ class MainWindow:
         @param  widget: gtk.Widget that emits the signal
 
         """
-        dlg = gtk.Dialog(title=_('Settings'), parent=None, flags=0)
-        dlg.set_has_separator(False)
+        dialog = gtk.Dialog()
+        dialog.set_title(_("Settings"))
+        dialog.add_button(_("Save"), gtk.RESPONSE_OK)
 
-        dlg.add_button(_("Save"), gtk.RESPONSE_OK)
+        notebook = gtk.Notebook()
+        notebook.set_show_tabs(False)
+        notebook.set_scrollable(False)
+        notebook.set_show_border(False)
 
-        hbox = gtk.HBox()
-
-        label = gtk.Label(_("Username"))
-        entry = hildon.Entry(fhsize)
-        entry.set_text(self.username)
-        hbox.pack_start(label, False, False, 0)
-        hbox.pack_start(entry, True, True, 0)
-        dlg.vbox.pack_start(hbox, True, True, 0)
-
-        hbox = gtk.HBox()
-        label = gtk.Label(_("Profile name"))
-        entry2 = hildon.Entry(fhsize)
-        entry2.set_text(self.profile)
-        hbox.pack_start(label, False, False, 0)
-        hbox.pack_start(entry2, True, True, 0)
-        dlg.vbox.pack_start(hbox, True, True, 0)
+        # General page #
+        vbox = gtk.VBox()
+        vbox.set_homogeneous(True)
 
         mapbtn = hildon.PickerButton(fhsize, horbtn)
         data = mapsources.values()
@@ -1545,23 +1636,107 @@ class MainWindow:
         mapbtn.set_title(_("Map source"))
         mapbtn.set_value(mapsources[int(self.mapsource)])
 
-        dlg.vbox.pack_start(mapbtn, True, True, 0)
+        vbox.pack_start(mapbtn, False, False, 0)
+
+        notebook.append_page(vbox, gtk.Label())
+
+        # 11870 page #
+        vbox = gtk.VBox()
+
+        hbox = gtk.HBox()
+        label = gtk.Label(_("Username"))
+        username11870 = hildon.Entry(fhsize)
+        username11870.set_text(self.username)
+        hbox.pack_start(label, False, False, 0)
+        hbox.pack_start(username11870, True, True, 0)
+        vbox.pack_start(hbox, True, True, 0)
+
+        hbox = gtk.HBox()
+        label = gtk.Label(_("Profile name"))
+        profile11870 = hildon.Entry(fhsize)
+        profile11870.set_text(self.profile)
+        hbox.pack_start(label, False, False, 0)
+        hbox.pack_start(profile11870, True, True, 0)
+        vbox.pack_start(hbox, True, True, 0)
 
         button = hildon.Button(fhsize, horbtn)
         button.set_label(_("Validate 11870.com"))
-        button.connect("clicked", self.validate_once, dlg)
-        dlg.vbox.pack_start(button, True, True, 0)
+        button.connect("clicked", self.validate_once, dialog)
+        vbox.pack_start(button, True, True, 0)
 
-        dlg.show_all()
-        result = dlg.run()
+        notebook.append_page(vbox, gtk.Label())
+
+        # Share page #
+        vbox = gtk.VBox()
+
+        label = gtk.Label("Twitter")
+        vbox.pack_start(label, True, True, 0)
+
+        hbox = gtk.HBox()
+        label = gtk.Label("Username")
+        userTwitter = hildon.Entry(fhsize)
+        userTwitter.set_text(self.twittuser)
+        hbox.pack_start(label, False, False, 0)
+        hbox.pack_start(userTwitter, True, True, 0)
+        vbox.pack_start(hbox, True, True, 0)
+
+        hbox = gtk.HBox()
+        label = gtk.Label(_("Password"))
+        passTwitter = hildon.Entry(fhsize)
+        passTwitter.set_text(self.twittpass)
+        hbox.pack_start(label, False, False, 0)
+        hbox.pack_start(passTwitter, True, True, 0)
+        vbox.pack_start(hbox, True, True, 0)
+
+        notebook.append_page(vbox, gtk.Label())
+
+        # Buttons #
+        dialog.vbox.pack_start(notebook, True, True, 0)
+
+        hbox = gtk.HBox()
+
+        descbutton = hildon.GtkRadioButton(fhsize)
+        descbutton.set_mode(False)
+        descbutton.set_active(True)
+        descbutton.set_label(_('General'))
+        descbutton.connect("toggled", self.change_tab, notebook, 0)
+        hbox.pack_start(descbutton, True, True, 0)
+
+        button = hildon.GtkRadioButton(fhsize)
+        button.set_mode(False)
+        button.set_active(True)
+        button.set_label('11870.com')
+        button.set_group(descbutton)
+        button.connect("toggled", self.change_tab, notebook, 1)
+        hbox.pack_start(button, True, True, 0)
+
+        button = hildon.GtkRadioButton(fhsize)
+        button.set_mode(False)
+        button.set_label(_('Share'))
+        button.set_group(descbutton)
+        button.connect("clicked", self.change_tab, notebook, 2)
+        hbox.pack_start(button, True, True, 0)
+
+        dialog.vbox.pack_start(hbox, False, False, 0)
+
+        dialog.show_all()
+        result = dialog.run()
         if result == gtk.RESPONSE_OK:
-            self.username = entry.get_text()
-            self.profile = entry2.get_text()
+            self.username = username11870.get_text()
+            self.profile = profile11870.get_text()
             mapname = mapbtn.get_value()
             self.mapsource = mapsources.keys()[mapsources.values().index(mapname)]
-            self.settings.save_config(str(self.mapsource), self.username, self.authtoken, self.profile)
+            self.twittuser = userTwitter.get_text()
+            self.twittpass = passTwitter.get_text()
 
-        dlg.destroy()
+            self.settings.save_config(str(self.mapsource), self.username,
+                                    self.authtoken, self.profile,
+                                    self.twittuser, self.twittpass)
+
+        dialog.destroy()
+
+    def change_tab(self, widget, notebook, number):
+        notebook.set_current_page(number)
 
     def create_selector(self, data):
         """Create a hildon.TouchSelector with some data in.
@@ -1601,6 +1776,24 @@ class MainWindow:
         #print args
         p = subprocess.Popen(args)
 
+
+    def show_help(self, widget):
+        """Launch the webbrowser and show the documentation webpage
+
+        @type   widget: gtk.Widget
+        @param  widget: gtk.Widget that emits the signal
+
+        """
+        import dbus
+        bus = dbus.SystemBus()
+        proxy = bus.get_object("com.nokia.osso_browser",
+                                "/com/nokia/osso_browser/request")
+        iface = dbus.Interface(proxy, 'com.nokia.osso_browser')
+
+        url = "http://cusl4-cservices.forja.rediris.es/documentation.html"
+        iface.open_new_window(url)
+
+
 #TODO: make this in the same class, easier to get data??
 
 class GPS:
@@ -1618,6 +1811,8 @@ class GPS:
 
         self.control.connect("error-verbose", self.on_error)
         self.device.connect("changed", self.on_changed, self.control)
+        self.has_fix = False
+        self.latlon = [0.0, 0.0]
 
         #Start the GPS system
         gobject.idle_add(self.start_location, self.control)
@@ -1646,7 +1841,10 @@ class GPS:
             return
         if device.fix:
             if device.fix[1] & location.GPS_DEVICE_LATLONG_SET:
-                print "lat = %f, long = %f" % device.fix[4:6]
+                #print "lat = %f, long = %f" % device.fix[4:6]
+                print 'got GPS fix'
+                self.has_fix = True
+                self.latlon = device.fix[4:6]
                 control.stop()
 
     def start_location(self, control):
@@ -1674,8 +1872,6 @@ class GPS:
         #print "sats: ", self.device.satellites
         #print "cellinfo: ", self.device.cell_info
 
-        #FIXME: solo cuando hay un fix
-
         lat, lon = self.device.fix[4:6]
         return lat, lon
 
@@ -1739,8 +1935,9 @@ class About:
         textview.set_cursor_visible(False)
         textview.set_wrap_mode(gtk.WRAP_WORD)
         text = ("%s Daniel Martin Yerga (dyerga@gmail.com)\n"
+        "%s Andrew Black"
         "\n"
-        "%s:") % (_("Written by"), _("Translations"))
+        "%s:") % (_("Written by"), _("Icon designed by"), _("Translations"))
         textview.get_buffer().set_text(text)
 
         parea = hildon.PannableArea()
