@@ -115,13 +115,13 @@ class API:
         #It's possible search w/o authentication to 11870.com
         #But it needs a different URL
         if self.auth:
-            print self.authtoken, self.username
+            #print self.authtoken, self.username
             h.add_credentials(self.username, self.authtoken)
             request, content = h.request(SEARCHURLAUTH % (text, page))
         else:
             request, content = h.request(SEARCHURL % (text, APPKEY, self.authsign, page))
 
-        print content, '\n\n'
+        #print content, '\n\n'
         feed = feedparser.parse(content)
         #print feed
 
